@@ -63,16 +63,14 @@ class SlugBranchGetter(object):
                 branch = repo.lookup_branch(full_name)
             return branch
 
-    def __call__(self, repo, issue):
+    def __call__(self, repo, issue, create=True):
         branch = self.get_branch(repo, issue)
 
         if not branch:
             # No local branches found, fetch remote
             with taskstatus('Fetching from "{}"', self.remote_name) as ts:
-                for remote in repo.remotes:
-                    if remote.name == self.remote_name:
-                        break
-                else:
+                remote = repo.lookup_remote(self.remote_name)
+                if not remote:
                     ts.abort('Remote "{}" not found', self.remote_name)
 
                 remote.credentials = self.remote_credentials
@@ -87,7 +85,7 @@ class SlugBranchGetter(object):
                 if branch:
                     ts.ok('Created new working branch based on existing '
                           'remote branch')
-                else:
+                elif create:
                     _, full_name = self.get_branch_name(self.prefix, issue)
 
                     base = self.get_base_branch(repo)
@@ -98,3 +96,10 @@ class SlugBranchGetter(object):
                     ts.ok('Created new working branch')
 
         return branch
+
+
+class Repository(pygit2.Repository):
+    def lookup_remote(self, name):
+        for remote in self.remotes:
+            if remote.name == name:
+                return remote
