@@ -7,18 +7,17 @@ import configparser
 
 
 PACKAGE = 'lancet'
-
+LOCAL_CONFIG = '.{}'.format(PACKAGE)
 DEFAULT_CONFIG = os.path.join(os.path.dirname(__file__),
                               'default-settings.ini')
 SYSTEM_CONFIG = '/etc/{0}/{0}.conf'.format(PACKAGE)
-USER_CONFIG = os.path.expanduser('~/.{}'.format(PACKAGE))
-PROJECT_CONFIG = os.path.join(os.path.realpath('.'), '.{}'.format(PACKAGE))
+USER_CONFIG = os.path.expanduser(os.path.join('~', LOCAL_CONFIG))
+PROJECT_CONFIG = os.path.join(os.path.realpath('.'), LOCAL_CONFIG)
 
 DEFAULT_FILES = [
     DEFAULT_CONFIG,
     SYSTEM_CONFIG,
     USER_CONFIG,
-    PROJECT_CONFIG,
 ]
 
 
@@ -44,6 +43,6 @@ def load_config(path=None, defaults=None):
 
     if path:
         with open(path) as fh:
-            config.readfp(fh)
+            config.read_file(fh)
 
     return config
